@@ -38,18 +38,28 @@
 #include <stdlib.h>
 #include <string.h>
 
+enum {
+  PROP_0,
+  PROP_ORIENTATION
+};
+
 #define EOG_THUMB_VIEW_SPACING 0
 
 #define EOG_THUMB_VIEW_GET_PRIVATE(object)				\
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_THUMB_VIEW, EogThumbViewPrivate))
 
-G_DEFINE_TYPE (EogThumbView, eog_thumb_view, GTK_TYPE_ICON_VIEW);
+static void eog_thumb_view_init (EogThumbView *thumbview);
 
 static EogImage* eog_thumb_view_get_image_from_path (EogThumbView      *thumbview,
 						     GtkTreePath       *path);
 
 static void      eog_thumb_view_popup_menu          (EogThumbView      *widget,
 						     GdkEventButton    *event);
+static void      eog_thumb_view_update_columns      (EogThumbView *view);
+
+G_DEFINE_TYPE_WITH_CODE (EogThumbView, eog_thumb_view, GTK_TYPE_ICON_VIEW,
+			G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL));
+
 static gboolean
 thumbview_on_query_tooltip_cb (GtkWidget  *widget,
 			       gint        x,
@@ -75,6 +85,11 @@ struct _EogThumbViewPrivate {
 	GtkWidget *menu;  /* a contextual menu for thumbnails */
 	GtkCellRenderer *pixbuf_cell;
 	gint visible_range_changed_id;
+
+	GtkOrientation orientation;
+	gint n_images;
+	gulong image_add_id;
+	gulong image_removed_id;
 };
 
 /* Drag 'n Drop */
@@ -147,15 +162,68 @@ static void
 eog_thumb_view_dispose (GObject *object)
 {
 	EogThumbViewPrivate *priv = EOG_THUMB_VIEW (object)->priv;
+	GtkTreeModel *model;
 
 	if (priv->visible_range_changed_id != 0) {
 		g_source_remove (priv->visible_range_changed_id);
 		priv->visible_range_changed_id = 0;
 	}
 
+	model = gtk_icon_view_get_model (GTK_ICON_VIEW (object));
+
+	if (model && priv->image_add_id != 0) {
+		g_signal_handler_disconnect (model, priv->image_add_id);
+		priv->image_add_id = 0;
+	}
+
+	if (model && priv->image_removed_id) {
+		g_signal_handler_disconnect (model, priv->image_removed_id);
+		priv->image_removed_id = 0;
+	}
+
 	G_OBJECT_CLASS (eog_thumb_view_parent_class)->dispose (object);
 }
 
+static void
+eog_thumb_view_get_property (GObject    *object,
+			     guint       prop_id,
+			     GValue     *value,
+			     GParamSpec *pspec)
+{
+	EogThumbView *view = EOG_THUMB_VIEW (object);
+
+	switch (prop_id)
+	{
+	case PROP_ORIENTATION:
+		g_value_set_enum (value, view->priv->orientation);
+		break;
+
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+eog_thumb_view_set_property (GObject      *object,
+			     guint         prop_id,
+			     const GValue *value,
+			     GParamSpec   *pspec)
+{
+	EogThumbView *view = EOG_THUMB_VIEW (object);
+
+	switch (prop_id)
+	{
+	case PROP_ORIENTATION:
+		view->priv->orientation = g_value_get_enum (value);
+		eog_thumb_view_update_columns (view);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 eog_thumb_view_class_init (EogThumbViewClass *class)
 {
@@ -163,6 +231,11 @@ eog_thumb_view_class_init (EogThumbViewClass *class)
 
 	gobject_class->constructed = eog_thumb_view_constructed;
 	gobject_class->dispose = eog_thumb_view_dispose;
+	gobject_class->get_property = eog_thumb_view_get_property;
+	gobject_class->set_property = eog_thumb_view_set_property;
+
+	g_object_class_override_property (gobject_class, PROP_ORIENTATION,
+	                                  "orientation");
 
 	g_type_class_add_private (class, sizeof (EogThumbViewPrivate));
 }
@@ -567,6 +640,9 @@ eog_thumb_view_init (EogThumbView *thumbview)
 	thumbview->priv = EOG_THUMB_VIEW_GET_PRIVATE (thumbview);
 
 	thumbview->priv->visible_range_changed_id = 0;
+	thumbview->priv->image_add_id = 0;
+	thumbview->priv->image_removed_id = 0;
+
 }
 
 /**
@@ -586,6 +662,43 @@ eog_thumb_view_new (void)
 	return GTK_WIDGET (thumbview);
 }
 
+static void
+eog_thumb_view_update_columns (EogThumbView *view)
+{
+	EogThumbViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_THUMB_VIEW (view));
+
+	priv = view->priv;
+
+	if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+			gtk_icon_view_set_columns (GTK_ICON_VIEW (view),
+			                           priv->n_images);
+}
+
+static void
+eog_thumb_view_row_inserted_cb (GtkTreeModel    *tree_model,
+                                GtkTreePath     *path,
+                                GtkTreeIter     *iter,
+                                EogThumbView    *view)
+{
+	EogThumbViewPrivate *priv = view->priv;
+
+	priv->n_images++;
+	eog_thumb_view_update_columns (view);
+}
+
+static void
+eog_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
+                               GtkTreePath     *path,
+                               EogThumbView    *view)
+{
+	EogThumbViewPrivate *priv = view->priv;
+
+	priv->n_images--;
+	eog_thumb_view_update_columns (view);
+}
+
 /**
  * eog_thumb_view_set_model:
  * @thumbview: A #EogThumbView.
@@ -599,13 +712,44 @@ void
 eog_thumb_view_set_model (EogThumbView *thumbview, EogListStore *store)
 {
 	gint index;
+	EogThumbViewPrivate *priv;
+	GtkTreeModel *existing;
 
 	g_return_if_fail (EOG_IS_THUMB_VIEW (thumbview));
 	g_return_if_fail (EOG_IS_LIST_STORE (store));
 
+	priv = thumbview->priv;
+
+	existing = gtk_icon_view_get_model (GTK_ICON_VIEW (thumbview));
+
+	if (existing != NULL) {
+		if (priv->image_add_id != 0) {
+			g_signal_handler_disconnect (existing,
+			                             priv->image_add_id);
+		}
+		if (priv->image_removed_id != 0) {
+			g_signal_handler_disconnect (existing,
+			                             priv->image_removed_id);
+
+		}
+	}
+
+	priv->image_add_id = g_signal_connect (G_OBJECT (store), "row-inserted",
+	                            G_CALLBACK (eog_thumb_view_row_inserted_cb),
+	                            thumbview);
+	priv->image_removed_id = g_signal_connect (G_OBJECT (store),
+	                             "row-deleted",
+	                             G_CALLBACK (eog_thumb_view_row_deleted_cb),
+	                             thumbview);
+
+	thumbview->priv->n_images = eog_list_store_length (store);
+
 	index = eog_list_store_get_initial_pos (store);
 
-	gtk_icon_view_set_model (GTK_ICON_VIEW (thumbview), GTK_TREE_MODEL (store));
+	gtk_icon_view_set_model (GTK_ICON_VIEW (thumbview),
+	                         GTK_TREE_MODEL (store));
+
+	eog_thumb_view_update_columns (thumbview);
 
 	if (index >= 0) {
 		GtkTreePath *path = gtk_tree_path_new_from_indices (index, -1);
